/***
* ==++==
*
* 
*
* ==--==
* =+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+
*
* cloudstorage.h
*
* This file contains cloud based storage classes.
* These classes are tested with the Dropbox REST APIs
*
* =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
****/
#ifndef CLOUD_STORAGE_H
#define CLOUD_STORAGE_H
#endif
#pragma once

#include <stdint.h>
#include <limits>
#include <string>
#include <vector>
#include <memory>
#include <unordered_map>
#include <tuple>

#include "pplxtasks.h"
#include "uri.h"
#include "asyncrt_utils.h"
#include "http_client.h"

namespace web { namespace storage {

/// <summary>
/// Association of a app consumer key and app secret
/// </summary>
class AppCredentials
{
public:
	AppCredentials()
	{
	}

	/// <summary>
	/// Creates storage credentials based on a storage account name and an azure key
	/// </summary>
	/// <param name="consumerKey">The consumer key associated with a DropBox App.</param>
	/// <param name="appSecret">The App Secret token.</param>
	AppCredentials(const std::wstring& token, const std::wstring& tokenSecret)
		: m_token(token), m_tokenSecret(tokenSecret)
	{
	}


	/// <summary>
	/// Returns the consumer key.
	/// </summary>
	/// <returns>The consumer key associated with the app</returns>
	const std::wstring& Token() const 
	{
		return m_token;
	}

	void SetToken(std::wstring value)
	{
		m_token = value;
	}

	/// <summary>
	/// Returns a reference to the non-base64 encoded app secret.
	/// </summary>
	/// <returns>The non-encoded secret for this app</returns>
	const std::wstring& TokenSecret() const
	{
		return m_tokenSecret; 
	}

	void SetTokenSecret(std::wstring value)
	{
		m_tokenSecret = value;
	}

private:
	std::wstring            m_token;
	std::wstring			m_tokenSecret;
};

}
}