﻿//
// MainPage.xaml.cpp
// Implementation of the MainPage class.
//

#include "pch.h"
#include "MainPage.xaml.h"
#include <collection.h>

using namespace XamlDropboxApp;

using namespace Platform;
using namespace Windows::Foundation;
using namespace Windows::Foundation::Collections;
using namespace Windows::UI::Xaml;
using namespace Windows::UI::Xaml::Controls;
using namespace Windows::UI::Xaml::Controls::Primitives;
using namespace Windows::UI::Xaml::Data;
using namespace Windows::UI::Xaml::Input;
using namespace Windows::UI::Xaml::Media;
using namespace Windows::UI::Xaml::Navigation;
using namespace Windows::Security::Authentication::OnlineId;
using namespace Windows::Security::Authentication::Web;
using namespace Platform::Collections;

using namespace Authentication;
using namespace web::storage;
using namespace web::http;
using namespace web::http::client;
using namespace concurrency;
using namespace concurrency::streams;
using namespace pplx;

// The Blank Page item template is documented at http://go.microsoft.com/fwlink/?LinkId=234238

MainPage::MainPage()
{
	InitializeComponent();
	UploadFileBtn->IsEnabled = false;
}

/// <summary>
/// Invoked when this page is about to be displayed in a Frame.
/// </summary>
/// <param name="e">Event data that describes how this page was reached.  The Parameter
/// property is typically used to configure the page.</param>
void MainPage::OnNavigatedTo(NavigationEventArgs^ e)
{
	(void) e;	// Unused parameter
}


void MainPage::SignInBtnClicked(Platform::Object^ sender, RoutedEventArgs^ e)
{

	if ((ConsumerKey->Text == nullptr) || (ConsumerSecret->Text == nullptr))
	{
		using namespace Windows::UI::Popups;
		auto msgDlg = ref new MessageDialog("Please check the input for the Consumer Key and/or Consumer Secret tokens");
		msgDlg->ShowAsync();
	}
	
	m_dispatcher = Windows::UI::Core::CoreWindow::GetForCurrentThread()->Dispatcher;

	m_creds = std::make_shared<AppCredentials>();
	m_authenticator = ref new OnlineIdAuthenticator();
	consumerKey = ConsumerKey->Text->Data();
	consumerSecret = ConsumerSecret->Text->Data();

	ConsumerKey->Text = nullptr;
	ConsumerSecret->Text = nullptr;

	OAuthLoginAsync(m_creds).then([this] 
	{	
		m_authurl = DropBoxAuthorizeURI;	
		m_authurl += utility::conversions::to_string_t(this->m_creds->Token());
		m_authurl += L"&oauth_callback=";
		m_authurl += WebAuthenticationBroker::GetCurrentApplicationCallbackUri()->AbsoluteUri->Data();

		auto action = m_dispatcher->RunAsync(
					Windows::UI::Core::CoreDispatcherPriority::Normal,
					ref new Windows::UI::Core::DispatchedHandler([this]()
					{
						auto beginUri = ref new Uri(ref new String(m_authurl.c_str()));
						
						task<WebAuthenticationResult^> authTask(WebAuthenticationBroker::AuthenticateAsync(WebAuthenticationOptions::None, beginUri));
		
						authTask.then([this](WebAuthenticationResult^ result)
						{
							String^ statusString;
							switch(result->ResponseStatus)
							{
							case WebAuthenticationStatus::Success:
								{
									auto actionEnable = m_dispatcher->RunAsync(
										Windows::UI::Core::CoreDispatcherPriority::Normal,
										ref new Windows::UI::Core::DispatchedHandler([this]()
									{
										UploadFileBtn->IsEnabled = true;
									}));									
								}
							}
						});						
					}));	
	});
}

task<void> MainPage::OAuthLoginAsync(std::shared_ptr<AppCredentials>& creds)
{	
    uri url(DropBoxRequestTokenURI);
    std::shared_ptr<OAuth> oAuthObj = std::make_shared<OAuth>();		
    auto signatureParams = 
		oAuthObj->CreateOAuthSignedParameters(url.to_string(),
		L"GET",
		NULL,
		consumerKey,
		consumerSecret
		);
		
    std::wstring sb = oAuthObj->OAuthBuildSignedHeaders(url);	
    http_client client(sb);	
	
    // Make the request and asynchronously process the response. 		
    return client.request(methods::GET)
           .then([&creds](http_response response)
    {			
        if(response.status_code() != status_codes::OK)
        {		    
            // Handle error cases... 
            return pplx::task_from_result();
        }
        
        // Perform actions here reading from the response stream... 
        // In this example, we parse the response body and 
		// extract the token and token secret.
        istream bodyStream = response.body();
        container_buffer<std::string> inStringBuffer;
        return bodyStream.read_to_end(inStringBuffer)
               .then([inStringBuffer, &creds](pplx::task<size_t> previousTask)
        {
			UNREFERENCED_PARAMETER(previousTask);
			const std::string &text = inStringBuffer.collection();

            // convert the response text to a wide-character string and then extract the tokens.
            std::wstring_convert<std::codecvt_utf8_utf16<wchar_t>, wchar_t> utf16conv;
            std::wostringstream ss;
            std::vector<std::wstring> parts;
            ss << utf16conv.from_bytes(text.c_str()) << std::endl;
            Split(ss.str(), parts, '&', false);
            unsigned pos = parts[1].find('=');
            std::wstring token = parts[1].substr(pos + 1, 16);
            
            pos = parts[0].find('=');
            std::wstring tokenSecret = parts[0].substr(pos + 1);
            creds->SetToken(token);
            creds->SetTokenSecret(tokenSecret);
        });
    });
}

task<void> MainPage::OAuthAcquireTokenAsync(std::shared_ptr<AppCredentials>& creds)
{
    uri url(DropBoxAccessTokenURI);

    std::shared_ptr<OAuth> oAuthObj = std::make_shared<OAuth>();	
	
    auto signatureParams = 
		oAuthObj->CreateOAuthSignedParameters(url.to_string(), 
		L"GET", 
		NULL,
		consumerKey,
		consumerSecret,
		creds->Token(),
		creds->TokenSecret()
		);
	
	
    std::wstring sb = oAuthObj->OAuthBuildSignedHeaders(url);
	
    http_client client(sb);	
	
    //// Make the request and asynchronously process the response. 		
    return client.request(methods::GET).then([&creds](http_response response)
    {				
        if(response.status_code() != status_codes::OK)
        {
		    auto stream = response.body();		
		    container_buffer<std::string> inStringBuffer;
		    return stream.read_to_end(inStringBuffer)
				.then([inStringBuffer](pplx::task<size_t> previousTask)
		    {
				UNREFERENCED_PARAMETER(previousTask);
			    const std::string &text = inStringBuffer.collection();

			    // convert the response text to a wide-character string.
			    std::wstring_convert<std::codecvt_utf8_utf16<wchar_t>, wchar_t> utf16conv;
			    std::wostringstream ss;				
			    ss << utf16conv.from_bytes(text.c_str()) << std::endl;				
			    OutputDebugString(ss.str().data());
				// Handle error cases... 
				return pplx::task_from_result();
		    });            
        }

        // Perform actions here reading from the response stream...         
	    istream bodyStream = response.body();
        container_buffer<std::string> inStringBuffer;
	    return bodyStream.read_to_end(inStringBuffer)
			.then([inStringBuffer, &creds](pplx::task<size_t> previousTask)
        {
			UNREFERENCED_PARAMETER(previousTask);
            const std::string &text = inStringBuffer.collection();

            // convert the response text to a wide-character string.
            std::wstring_convert<std::codecvt_utf8_utf16<wchar_t>, wchar_t> utf16conv;
            std::wostringstream ss;
		    std::vector<std::wstring> parts;
            ss << utf16conv.from_bytes(text.c_str()) << std::endl;
		    Split(ss.str(), parts, '&', false);
		    unsigned pos = parts[1].find('=');
		    std::wstring token = parts[1].substr(pos + 1, 16);

		    pos = parts[0].find('=');
		    std::wstring tokenSecret = parts[0].substr(pos + 1);			
		    creds->SetToken(token);
		    creds->SetTokenSecret(tokenSecret);
        });

    });

}

task<void> MainPage::UploadFileToDropBoxAsync(std::shared_ptr<AppCredentials>& creds)
{
    using concurrency::streams::file_stream;
    using concurrency::streams::basic_istream;

    uri url(DropBoxFileUploadURI);
    std::shared_ptr<OAuth> oAuthObj = std::make_shared<OAuth>();	
	
    auto signatureParams = 
		oAuthObj->CreateOAuthSignedParameters(url.to_string(),
		L"PUT",
		NULL,
		consumerKey,
		consumerSecret,
		creds->Token(),
		creds->TokenSecret()
		);	
    
	std::wstring sb = oAuthObj->OAuthBuildSignedHeaders(url);
    return file_stream<unsigned char>::open_istream(this->m_fileToUpload)
           .then([this, sb, url](pplx::task<basic_istream<unsigned char>> previousTask)
    {
        try
        {
            auto fileStream = previousTask.get();
            //get the content length, used to set the Content-Length property
            fileStream.seek(0, std::ios::end);
            auto length = static_cast<size_t>(fileStream.tell());
            fileStream.seek(0, 0);

            // Make HTTP request with the file stream as the body.						
            http_request req;			
            http_client client(sb);		
            req.set_body(fileStream, length);				
            req.set_method(methods::PUT);			
            return client.request(req)
                    .then([this, fileStream](pplx::task<http_response> previousTask)
            {
	            fileStream.close();
	            std::wostringstream ss;
	            try
	            {
		            auto response = previousTask.get();
		            auto body = response.body();		
		            // log response success code
					ss << L"Server returned status code "
						<< response.status_code() << L"."
						<< std::endl;
					OutputDebugString(ss.str().data());
					if (response.status_code() == web::http::status_codes::OK)
					{
						auto action = m_dispatcher->RunAsync(
						Windows::UI::Core::CoreDispatcherPriority::Normal,
						ref new Windows::UI::Core::DispatchedHandler([this]()
						{
							using namespace Windows::UI::Popups;
							auto msgDlg = ref new MessageDialog("File uploaded successfully to Dropbox");
							msgDlg->ShowAsync();
						}));
		
						
					}
	            }
	            catch (const http_exception& e)
	            {
		            ss << e.what() << std::endl;
					OutputDebugString(ss.str().data());
	            }				
            });            
        }		
        catch (const std::system_error& e)
        {
            // log any errors here
            // Return an empty task. 
			std::wostringstream ss;
			ss << e.what() << std::endl;
			OutputDebugString(ss.str().data());
            return pplx::task_from_result();
        }
    });
}

void MainPage::UploadFileBtnClicked(Platform::Object^ sender, RoutedEventArgs^ e)
{
	using namespace Windows::Storage::Pickers;
	using namespace Windows::Storage;
	auto picker = ref new FileOpenPicker();
	picker->SuggestedStartLocation = PickerLocationId::DocumentsLibrary;
	picker->FileTypeFilter->Append(".txt");

	task<StorageFile^> (picker->PickSingleFileAsync()).then([this](StorageFile^ selectedFile)
	{
		m_fileToUpload = selectedFile;
		OAuthAcquireTokenAsync(m_creds).then([this](){
			UploadFileToDropBoxAsync(m_creds);
		});
		
	});	
}

